@@ -16,7 +16,7 @@
 mod cache;
 mod pipeline;
 
-/// Contains all needed structs and enums for inserting and styling text.
+/// Contains all needed objects for inserting and styling text.
 /// Directly taken from glyph_brush.
 ///
 /// Look into [glyph_brush_layout docs](https://docs.rs/glyph_brush_layout/latest/glyph_brush_layout/#enums)
@@ -30,6 +30,17 @@ pub mod section {
     };
 }
 
+/// Contains all needed objects for font managing.
+/// Directly taken from glyph_brush.
+///
+/// Look into [glyph_brush_font docs](https://docs.rs/glyph_brush/latest/glyph_brush/ab_glyph/index.html)
+/// for the real, detailed, documentation.
+/// - If anything is missing open an issue on github and I'll add it.
+pub mod font {
+    #[doc(hidden)]
+    pub use glyph_brush::ab_glyph::{Font, FontArc, FontRef, InvalidFont};
+}
+
 use glyph_brush::{
     ab_glyph::{Font, FontArc, FontRef, InvalidFont},
     BrushAction, DefaultSectionHasher, Extra, Section,
@@ -61,6 +72,9 @@ pub struct ScissorRegion {
     pub out_height: u32,
 }
 
+/// Represents a two dimensional array matrix with 4x4 dimensions.
+pub type Matrix = [[f32; 4]; 4];
+
 impl ScissorRegion {
     /// Checks if the region is contained in surface bounds at all.
     pub(crate) fn is_contained(&self) -> bool {
@@ -92,12 +106,12 @@ impl ScissorRegion {
 ///
 /// Used for queuing and rendering text with
 /// [`TextBrush::draw`] and [`TextBrush::draw_custom`].
-pub struct TextBrush<Depth, F = FontArc, H = DefaultSectionHasher> {
+pub struct TextBrush<F = FontArc, H = DefaultSectionHasher> {
     inner: glyph_brush::GlyphBrush<Vertex, Extra, F, H>,
-    pipeline: Pipeline<Depth>,
+    pipeline: Pipeline,
 }
 
-impl<Depth, F, H> TextBrush<Depth, F, H>
+impl<F, H> TextBrush<F, H>
 where
     F: Font + Sync,
     H: std::hash::BuildHasher,
@@ -240,47 +254,45 @@ where
     /// (`cross product`-ing).
     pub fn update_matrix<M>(&mut self, matrix: M, queue: &wgpu::Queue)
     where
-        M: Into<[[f32; 4]; 4]>,
+        M: Into<Matrix>,
     {
         self.pipeline.update_matrix(matrix.into(), queue);
     }
-}
 
-impl<F, H> TextBrush<wgpu::DepthStencilState, F, H>
-where
-    F: Font + Sync,
-    H: std::hash::BuildHasher,
-{
-    /// Available if *TextBrush* was built with [`BrushBuilder::with_depth_testing()`].
     /// Resizes depth texture to provided dimensions.
-    ///
+    /// 
+    /// Required if [`BrushBuilder::with_depth_testing()`] is set to `true`.
+    /// 
     /// Should be used every time the window (`wgpu::SurfaceConfiguration`) is resized.
-    /// If not used when needed, your program will crash with wgpu error.
+    /// If not used when needed, program will crash with _wgpu error_.
+    /// 
+    /// If used while [`BrushBuilder::with_depth_testing()`] is set to `false`
+    /// nothing will happen.
     #[inline]
     pub fn resize_depth(&mut self, device: &wgpu::Device, width: u32, height: u32) {
-        self.pipeline.update_depth(device, (width, height));
+        if self.pipeline.depth_texture_view.is_some() {
+            self.pipeline.update_depth(device, (width, height));
+        }
     }
 }
 
 /// Builder for [`TextBrush`].
-pub struct BrushBuilder<Depth, F, H = DefaultSectionHasher> {
+pub struct BrushBuilder<F, H = DefaultSectionHasher> {
     inner: glyph_brush::GlyphBrushBuilder<F, H>,
-    matrix: Option<[[f32; 4]; 4]>,
-    phantom: std::marker::PhantomData<Depth>,
+    depth_testing: Option<wgpu::DepthStencilState>,
+    matrix: Option<Matrix>,
 }
 
-impl BrushBuilder<(), ()> {
+impl BrushBuilder<()> {
     /// Creates a [`BrushBuilder`] with [`Font`].
     #[inline]
-    pub fn using_font<F: Font>(font: F) -> BrushBuilder<(), F> {
+    pub fn using_font<F: Font>(font: F) -> BrushBuilder<F> {
         BrushBuilder::using_fonts(vec![font])
     }
 
     /// Creates a [`BrushBuilder`] with font byte data.
     #[inline]
-    pub fn using_font_bytes(
-        data: &[u8],
-    ) -> Result<BrushBuilder<(), FontRef>, InvalidFont> {
+    pub fn using_font_bytes(data: &[u8]) -> Result<BrushBuilder<FontRef>, InvalidFont> {
         let font = FontRef::try_from_slice(data)?;
         Ok(BrushBuilder::using_fonts(vec![font]))
     }
@@ -289,22 +301,22 @@ impl BrushBuilder<(), ()> {
     #[inline]
     pub fn using_font_bytes_vec(
         data: &[u8],
-    ) -> Result<BrushBuilder<(), FontRef>, InvalidFont> {
+    ) -> Result<BrushBuilder<FontRef>, InvalidFont> {
         let font = FontRef::try_from_slice(data)?;
         Ok(BrushBuilder::using_fonts(vec![font]))
     }
 
     /// Creates a [`BrushBuilder`] with multiple [`Font`].
-    pub fn using_fonts<F: Font>(fonts: Vec<F>) -> BrushBuilder<(), F> {
+    pub fn using_fonts<F: Font>(fonts: Vec<F>) -> BrushBuilder<F> {
         BrushBuilder {
             inner: glyph_brush::GlyphBrushBuilder::using_fonts(fonts),
+            depth_testing: None,
             matrix: None,
-            phantom: std::marker::PhantomData::<()>,
         }
     }
 }
 
-impl<Depth, F, H> BrushBuilder<Depth, F, H>
+impl<F, H> BrushBuilder<F, H>
 where
     F: Font,
     H: std::hash::BuildHasher,
@@ -316,100 +328,61 @@ where
     /// To update the render matrix use [`TextBrush::update_matrix()`].
     pub fn with_matrix<M>(mut self, matrix: M) -> Self
     where
-        M: Into<[[f32; 4]; 4]>,
+        M: Into<Matrix>,
     {
         self.matrix = Some(matrix.into());
         self
     }
-}
 
-impl<F, H> BrushBuilder<(), F, H>
-where
-    F: Font,
-    H: std::hash::BuildHasher,
-{
-    /// If called depth testing will be enabled. By default depth testing
-    /// is disabled.
-    ///
+    /// Defaults to `false`. If set to true all text will be depth tested.
+    /// 
     /// For each section, depth can be set by modifying
     /// the z coordinate ([`OwnedText::with_z()`]).
     ///
     /// `z` coordinate should be in range
     ///  0.0 - 1.0 not including 1.0.
-    pub fn with_depth_testing(self) -> BrushBuilder<wgpu::DepthStencilState, F, H> {
-        BrushBuilder {
-            inner: self.inner,
-            matrix: self.matrix,
-            phantom: std::marker::PhantomData::<wgpu::DepthStencilState>,
-        }
-    }
-
-    /// Builds a [`TextBrush`] consuming [`BrushBuilder`].
-    pub fn build(
-        self,
-        device: &wgpu::Device,
-        render_format: wgpu::TextureFormat,
-        dimensions: (u32, u32),
-    ) -> TextBrush<(), F, H> {
-        let inner = self.inner.build();
-
-        let matrix = if let Some(matrix) = self.matrix {
-            matrix
+    pub fn with_depth_testing(mut self, test: bool) -> BrushBuilder<F, H> {
+        if test {
+            self.depth_testing = Some(wgpu::DepthStencilState {
+                format: Pipeline::DEPTH_FORMAT,
+                depth_write_enabled: true,
+                depth_compare: wgpu::CompareFunction::Less,
+                stencil: wgpu::StencilState::default(),
+                bias: wgpu::DepthBiasState::default(),
+            });
         } else {
-            ortho(dimensions.0 as f32, dimensions.1 as f32)
-        };
-
-        let pipeline = Pipeline::new(
-            device,
-            render_format,
-            None,
-            inner.texture_dimensions(),
-            matrix,
-        );
-
-        TextBrush { inner, pipeline }
+            self.depth_testing = None;
+        }
+        self
     }
-}
 
-impl<F, H> BrushBuilder<wgpu::DepthStencilState, F, H>
-where
-    F: Font,
-    H: std::hash::BuildHasher,
-{
     /// Builds a [`TextBrush`] with depth testing consuming [`BrushBuilder`].
     ///
     /// To use this build method call [`Self::with_depth_testing()`].
     pub fn build(
         self,
         device: &wgpu::Device,
-        render_format: wgpu::TextureFormat,
-        dimensions: (u32, u32),
-    ) -> TextBrush<wgpu::DepthStencilState, F, H> {
+        config: &wgpu::SurfaceConfiguration,
+    ) -> TextBrush<F, H> {
         let inner = self.inner.build();
 
-        let matrix = if let Some(matrix) = self.matrix {
-            matrix
+        let matrix = if let Some(m) = self.matrix {
+            m
         } else {
-            ortho(dimensions.0 as f32, dimensions.1 as f32)
-        };
-
-        let depth = wgpu::DepthStencilState {
-            format: Pipeline::DEPTH_FORMAT,
-            depth_write_enabled: true,
-            depth_compare: wgpu::CompareFunction::Less,
-            stencil: wgpu::StencilState::default(),
-            bias: wgpu::DepthBiasState::default(),
+            ortho(config.width as f32, config.height as f32)
         };
 
+        let depth = self.depth_testing.is_some();
         let mut pipeline = Pipeline::new(
             device,
-            render_format,
-            Some(depth),
+            config.format,
+            self.depth_testing,
             inner.texture_dimensions(),
             matrix,
-        )
-        .with_depth();
-        pipeline.update_depth(device, dimensions);
+        );
+        if depth {
+            pipeline.update_depth(device, (config.width, config.height));
+        }
 
         TextBrush { inner, pipeline }
     }
@@ -417,7 +390,7 @@ where
 
 /// Creates an orthographic matrix with given dimensions `width` and `height`.
 #[rustfmt::skip]
-pub fn ortho(width: f32, height: f32) -> [[f32; 4]; 4] {
+pub fn ortho(width: f32, height: f32) -> Matrix {
     [
         [2.0 / width, 0.0,          0.0, 0.0],
         [0.0,        -2.0 / height, 0.0, 0.0],
